@@ -793,11 +793,14 @@ def _get_service_endpoints(name, format_str, public, internal, admin=None,
         public_port_key = 'ssl_port'
 
     endpoints['UNDERCLOUD_ENDPOINT_%s_PUBLIC' % upper_name] = (
-        format_str % (public_proto, public['host'], public[public_port_key]))
+        format_str % (public_proto, _wrap_ipv6(public['host']),
+                      public[public_port_key]))
     endpoints['UNDERCLOUD_ENDPOINT_%s_INTERNAL' % upper_name] = (
-        format_str % (internal_proto, internal['host'], internal['port']))
+        format_str % (internal_proto, _wrap_ipv6(internal['host']),
+                      internal['port']))
     endpoints['UNDERCLOUD_ENDPOINT_%s_ADMIN' % upper_name] = (
-        format_str % (internal_proto, admin['host'], admin['port']))
+        format_str % (internal_proto, _wrap_ipv6(admin['host']),
+                      admin['port']))
     return endpoints
 
 
@@ -904,7 +907,7 @@ def _generate_endpoints(instack_env):
             {'host': internal_host, 'port': 8041}),
         ('panko',
             '%s://%s:%d',
-            {'host': public_host, 'port': 8977, 'ssl_port': 13779},
+            {'host': public_host, 'port': 8977, 'ssl_port': 13977},
             {'host': internal_host, 'port': 8977}),
         ('mistral',
             '%s://%s:%d/v2',
@@ -1206,9 +1209,6 @@ def _generate_environment(instack_root):
     instack_env['PUBLIC_INTERFACE_IP'] = instack_env['LOCAL_IP']
     instack_env['LOCAL_IP'] = instack_env['LOCAL_IP'].split('/')[0]
     instack_env['LOCAL_IP_WRAPPED'] = _wrap_ipv6(instack_env['LOCAL_IP'])
-    if instack_env['UNDERCLOUD_SERVICE_CERTIFICATE']:
-        instack_env['UNDERCLOUD_SERVICE_CERTIFICATE'] = os.path.abspath(
-            instack_env['UNDERCLOUD_SERVICE_CERTIFICATE'])
     # We're not in a chroot so this doesn't make sense, and it causes weird
     # errors if it's set.
     if instack_env.get('DIB_YUM_REPO_CONF'):
@@ -1231,6 +1231,19 @@ def _generate_environment(instack_root):
         public_host = CONF.undercloud_public_host
         instack_env['UNDERCLOUD_SERVICE_CERTIFICATE'] = (
             '/etc/pki/tls/certs/undercloud-%s.pem' % public_host)
+    elif instack_env['UNDERCLOUD_SERVICE_CERTIFICATE']:
+        raw_value = instack_env['UNDERCLOUD_SERVICE_CERTIFICATE']
+        abs_cert = os.path.abspath(raw_value)
+        if abs_cert != raw_value:
+            home_dir = os.path.expanduser('~')
+            if os.getcwd() != home_dir and os.path.exists(abs_cert):
+                LOG.warning('Using undercloud_service_certificate from '
+                            'current directory, please use an absolute path '
+                            'to remove ambiguity')
+                instack_env['UNDERCLOUD_SERVICE_CERTIFICATE'] = abs_cert
+            else:
+                instack_env['UNDERCLOUD_SERVICE_CERTIFICATE'] = os.path.join(
+                    home_dir, raw_value)
 
     return instack_env
 
@@ -1254,12 +1267,20 @@ def _generate_init_data(instack_env):
     context = instack_env.copy()
 
     if CONF.hieradata_override:
-        hiera_entry = os.path.splitext(
-            os.path.basename(CONF.hieradata_override))[0]
+        data_file = CONF.hieradata_override
+        hiera_entry = os.path.splitext(os.path.basename(data_file))[0]
         dst = os.path.join('/etc/puppet/hieradata',
-                           os.path.basename(CONF.hieradata_override))
+                           os.path.basename(data_file))
+        if os.path.abspath(CONF.hieradata_override) != data_file:
+            # If we don't have an absolute path, compute it
+            data_file = os.path.join(os.path.expanduser('~'), data_file)
+
+        if not os.path.exists(data_file):
+            raise RuntimeError(
+                "Could not find hieradata_override file '%s'" % data_file)
+
         _run_command(['sudo', 'mkdir', '-p', '/etc/puppet/hieradata'])
-        _run_command(['sudo', 'cp', CONF.hieradata_override, dst])
+        _run_command(['sudo', 'cp', data_file, dst])
         _run_command(['sudo', 'chmod', '0644', dst])
     else:
         hiera_entry = ''
@@ -1426,7 +1447,7 @@ def _ensure_flavor(nova, existing, name, profile=None):
 
 
 def _ensure_node_resource_classes(ironic):
-    for node in ironic.node.list(limit=-1, fields=['uuid', 'resource_class']):
+    for node in ironic.node.list(limit=0, fields=['uuid', 'resource_class']):
         if node.resource_class:
             if node.resource_class != DEFAULT_NODE_RESOURCE_CLASS:
                 LOG.warning('Node %s is using a resource class %s instead '
@@ -1806,6 +1827,11 @@ def pre_upgrade():
     _run_live_command(args, name='systemctl stop')
     LOG.info('Services stopped successfully')
 
+    args = ['sudo', 'yum', 'install', '-y', 'ansible-pacemaker']
+    LOG.info('Installing Ansible Pacemaker module')
+    _run_live_command(args, name='install ansible')
+    LOG.info('Ansible pacemaker install completed successfully')
+
     args = ['sudo', 'yum', 'update', '-y']
     LOG.info('Updating full system')
     _run_live_command(args, name='yum update')
